#! /bin/sh
#
#   kuru.sh -- ¥¯¥ë¥¯¥ë
#

curtty=`tty | sed 's|/|-|g'`
if [ ! -w /tmp/kuru-$curtty.txt ]; then
	touch /tmp/kuru-$curtty.txt
fi
kuruchar=`sed -n '1p' < /tmp/kuru-$curtty.txt`
case "x${kuruchar}x" in
	x®¡x)   kuruchar='®¢' ;;
	x®¢x)   kuruchar='®£' ;;
	x®£x)   kuruchar='®¤' ;;
	x®¤x)   kuruchar='®¥' ;;
	x®¥x)   kuruchar='®¦' ;;
	x®¦x)   kuruchar='®§' ;;
	x®§x)   kuruchar='®¨' ;;
	x®¨x)   kuruchar='®ª' ;;
	x®ªx)   kuruchar='®«' ;;
	x®«x)   kuruchar='®¬' ;;
	x®¬x)   kuruchar='®­' ;;
	x®­x)   kuruchar='®®' ;;
	x®®x)   kuruchar='®¯' ;;
	x®¯x)   kuruchar='®°' ;;
	x®°x)   kuruchar='®±' ;;
	*®±x)   kuruchar='®²' ;;
	*®²x)   kuruchar='®©' ;;
	*)      kuruchar='®¡' ;;
esac
echo -n "$kuruchar "
echo "$kuruchar" > /tmp/kuru-$curtty.txt

exit 0

# EOF
# Local Variables:
# default-terminal-coding-system: iso-2022-jp
# End:
#
